# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
from Analyze import Analyze
import os.path


def DataOfRoc(standard_path="truth",test_path="test"):
    db=[]
    pos,db,temp=Analyze(standard_path,test_path)
    
    tp,fp = 0., 0.		
    for i in range(len(db)):
        tp += db[i][0]#wrong
        fp += db[i][1]#right
        
    rate=fp/pos
    print " checking wrong num=%f\n standard true num=%f\n recall rate=%f"%(tp,pos,rate)
    return tp,pos,rate
    

if __name__ == '__main__':
    DataOfRoc()
    


# -*- coding: utf-8 -*-
#import xml.dom.minidom 

def read(root):
    """
    读取一个xml文件的所有标签
    """
    Reframe=[]
    #filename=root.getElementsByTagName('filename')
    xmin=root.getElementsByTagName('xmin')
    xmax=root.getElementsByTagName('xmax')
    ymin=root.getElementsByTagName('ymin')
    ymax=root.getElementsByTagName('ymax')
    score=root.getElementsByTagName('score')
    rectnum = len(xmin)
    #n0=filename[0]
    #Name = n0.firstChild.data
    #Reframe.append(Name)
    for i in range(0,rectnum):#一个文件的多个框
        n1=xmin[i]
        n2=xmax[i]
        n3=ymin[i]
        n4=ymax[i]
        Xmin=Ymin=Xmax=Ymax=0
        Xmin = int(n1.firstChild.data)
        Xmax = int(n2.firstChild.data)
        Ymin = int(n3.firstChild.data)
        Ymax = int(n4.firstChild.data)
        if len(score)>=1:
            n5=score[i]
            Score = float(n5.firstChild.data)
            Reframe.append([Xmin,Ymin,Xmax,Ymax,Score])
        else:
            Reframe.append([Xmin,Ymin,Xmax,Ymax])
    return Reframe
               
if __name__ == '__main__':
    read()


# -*- coding: utf-8 -*-
import os
import os.path
import xml.dom.minidom

def xml2txt(pre_file='xml_file',after_file='txt_file'):
    
    files1 =os.listdir(pre_file)
    count=0
    for xmlFile in files1: #for every xmlfile
        if not os.path.isdir(xmlFile) and xmlFile.endswith('.xml'):#select xmlfile
            count+=1
            print "file %d is opened successfully"%count
            dom=xml.dom.minidom.parse(os.path.join(pre_file,xmlFile))
            
            root=dom.documentElement
 
            if len(root.getElementsByTagName('xmin'))==0:
                continue
            filename=root.getElementsByTagName('filename')
            score=root.getElementsByTagName('score')                         ###
            xmin=root.getElementsByTagName('xmin')
            xmax=root.getElementsByTagName('xmax')
            ymin=root.getElementsByTagName('ymin')
            ymax=root.getElementsByTagName('ymax')
            
            rectnum = len(xmin)
            
        
    #check datas
            n0=filename[0]
            Name = n0.firstChild.data

            meg = Name[:-4]+' '+str(rectnum)
            
            for i in range(0,rectnum):
                n1=xmin[i]
                n2=xmax[i]
                n3=ymin[i]
                n4=ymax[i]
                Xmin = n1.firstChild.data
                Xmax = n2.firstChild.data
                Ymin = n3.firstChild.data
                Ymax = n4.firstChild.data
                if len(score)>=1:
                    n5=score[i]
                    Score=n5.firstChild.data
                    meg +=' ' + str(Xmin)+' '+str(Ymin)+' '+str(Xmax)+' '+str(Ymax)+' '+str(Score)
                else:
                    meg +=' ' + str(Xmin)+' '+str(Ymin)+' '+str(Xmax)+' '+str(Ymax)
                    
            if not(os.path.exists(after_file)):
                os.mkdir(after_file)
            full_path = os.path.join(after_file ,Name)
            file = open(full_path,'w')             
            file.write(meg)
            file.close() 
    print "\n\nxml2txt is OK!"


xml2txt('xml_file','txt_file')


# -*- coding: utf-8 -*-
import os
import os.path
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

cnames = ('aquamarine','black','blue',
          'blueviolet','brown','burlywood','cadetblue','chartreuse','chocolate','coral','cornflowerblue',
          'crimson','cyan','darkblue','darkcyan','darkgoldenrod','darkgray','darkgreen','darkkhaki','darkmagenta',
          'darkolivegreen','darkorange','darkorchid','darkred','darkseagreen','darkslateblue','darkslategray',
          'darkturquoise','darkviolet','deeppink','deepskyblue','dimgray','firebrick','forestgreen',
          'fuchsia','gold','goldenrod','gray','green','hotpink','indianred',
          'indigo','lawngreen','lime','limegreen',
          'magenta','maroon','navy','olive','olivedrab',
          'orange','orchid','palevioletred','peru',
          'plum','purple','red','sienna','skyblue','slateblue','slategray','springgreen','steelblue',
          'tan','teal','thistle','turquoise','violet','yellow')

def drawpic(pre_file='file',image='image',titlestr='roc'):
    
    data=[]
    ab=[]
    aa=[]
    mux=0
    files1 =os.listdir(pre_file)
    for File in files1:#
        aa.append(File)
        tt=File[-4:]
        if tt!='.txt':
            break
        path=os.path.join(pre_file,File)
        f = open(path)              
        line = f.readline()    
        db=[]        
        while line:
            data=line.split()
            if data==[]:
                break
            data[0]=float(data[0])
            if data[0]>mux:
                mux=data[0]
            data[1]=float(data[1])
            data[2]=float(data[2])
            db.append(data) 
            line = f.readline() 
        db = sorted(db, key=lambda x:x[2], reverse=True)
        ab.append(db)
        print "file %s upload successfully" %File
    #return ab    
        
        
    for i in range(len(ab)):
        x = [_a[0] for _a in ab[i]]
        y = [_a[1] for _a in ab[i]]
        z = [_a[2] for _a in ab[i]]       
        plt.xlabel("False Count")
        plt.ylabel("True Positive Rate")
        plt.title(titlestr)
        plt.plot(x, y,color=cnames[i%69],label=aa[i][:-4])
        plt.plot(x, z,color=cnames[i%69])
    image+='.jpg'
    
    xmajorLocator   = MultipleLocator(mux/8)
    xminorLocator   = MultipleLocator(mux/16) 
    ymajorLocator   = MultipleLocator(0.2)
    yminorLocator   = MultipleLocator(0.05) 
    
    ax = plt.subplot()
    ax.xaxis.set_major_locator(xmajorLocator)
    ax.yaxis.set_major_locator(ymajorLocator)
    ax.xaxis.set_minor_locator(xminorLocator)
    ax.yaxis.set_minor_locator(yminorLocator)
    ax.xaxis.grid(True, which='major')
    ax.yaxis.grid(True, which='minor') 
    
    plt.legend(bbox_to_anchor=(0., 1.08, 1., .102), loc=2,ncol=3, mode='expand', borderaxespad=0.)
    plt.grid() 
    plt.grid(color='y' , linestyle='--')
    
    plt.savefig(image)
    plt.show() 
    print "Run successfully"
        


if __name__ == '__main__':
    drawpic()



# -*- coding: utf-8 -*-
import xml.dom.minidom 
import os.path
from read import read
from IOU import IOU
import os

def Analyze(standard_path,test_path):
    '''
    return:pos_num,ratio_info
    ratio_info:wrong,right,ratio/sore
    
    '''
    wrong,right,num_pos=0,0,0
    ratio=0.
    ratio_info=[]#返回值
    frame_stand=frame_test=[]#read result
    
    folder_stand=standard_path#文件夹路径
    folder_test=test_path
    
    files_stand = os.listdir(folder_stand)#文件列表  标x.s 标x.d 
    files_test = os.listdir(folder_test)           #测x.s 测x.d
    files_all=files_test                           #所有文件并集
    files_temp=[]                                  #只有标准文件的名字列表
    
    temp_y=0
    
    
    for File in files_stand:# all xmlfile
        path_test=os.path.join(folder_test,File)
        path_stand=path_test.replace(folder_test,folder_stand)
        if not os.path.isfile(path_test):
            files_all.append(File)# 测x.s 测x.d 标x.s
            files_temp.append(File)#标x.s
            
    for File in files_all:            
        if File in files_temp:#标x.s      每一个框都是一个实际正确 漏读
            path_stand=os.path.join(folder_stand,File)
            dom_stand=xml.dom.minidom.parse(path_stand)
            root_stand=dom_stand.documentElement
            frame_stand=read(root_stand)
            boxnum_stand=len(frame_stand)
            num_pos+=boxnum_stand
            num_info=len(frame_test[0])
            wrong=0
            right=0
            ratio=0.
            for i in range(0,boxnum_stand):
                if num_info==5:
                    temp_y=1
                    ratio_info.append([wrong,right,frame_test[i][4]])                       
                else:
                    ratio_info.append([wrong,right,ratio])
               
            
        else:                 #测x.s 测x.d
            path_test=os.path.join(folder_test,File)
            path_stand=path_test.replace(folder_test,folder_stand)
            
            if not os.path.isfile(path_stand): #测x.s         每一个框都是一个实际错误 误读(ratio=0)
                dom_test=xml.dom.minidom.parse(path_test)
                root_test=dom_test.documentElement
                frame_test=read(root_test)
                boxnum_test=len(frame_test)
                num_info=len(frame_test[0])
                wrong=1
                right=0
                ratio=0.
                for i in range(0,boxnum_test):
                    if num_info==5:
                         ratio_info.append([wrong,right,frame_test[i][4]])                        
                    else:
                         ratio_info.append([wrong,right,ratio])                       
                
            else:   #测x.d

                dom_test=xml.dom.minidom.parse(path_test) #测x.d
                root_test=dom_test.documentElement
                frame_test=read(root_test)
                boxnum_test=len(frame_test)
                
                num_info=len(frame_test[0])
                
                dom_stand=xml.dom.minidom.parse(path_stand)#标x.d
                root_stand=dom_stand.documentElement
                frame_stand=read(root_stand)
                boxnum_stand=len(frame_stand)
                
                num_pos+=boxnum_stand#标的框都是实际正确的
                for i in range(0,boxnum_test):
                    boxnum_stand=len(frame_stand)
                    wrong=right=0
                    if  boxnum_stand==0:
                        wrong=1
                        ratio=0.
                        if num_info==5:
                            ratio_info.append([wrong,right,frame_test[i][4]])            
                        else:
                            ratio_info.append([wrong,right,ratio])
                    else:
                        for j in range(0,boxnum_stand):
                            ratio=IOU(frame_test[i],frame_stand[j])  
                            if ratio>=0.5:
                                right=1
                                if num_info==5:
                                    ratio_info.append([wrong,right,frame_test[i][4]])            
                                else:
                                    ratio_info.append([wrong,right,ratio])
                                del frame_stand[j]
                                break
                            
                    if right==0 :
                        wrong=1
                        if num_info==5:
                            ratio_info.append([wrong,right,frame_test[i][4]])                   
                        else:
                            ratio_info.append([wrong,right,ratio])
                   

    #print num_pos
    #print len(ratio_info)
    
    return num_pos,ratio_info,temp_y
                            
                                
                                
                               
if __name__ == '__main__':
    Analyze('D:\\python_work\\bndbox_test\\roc-2.1.1\\ROC\\truth','D:\\python_work\\bndbox_test\\roc-2.1.1\\ROC\\test')                           

          
                        
                
                
                
            
        
        


# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
from Analyze import Analyze
import os.path
import random
from matplotlib.ticker import MultipleLocator, FormatStrFormatter


def roc(standard_path="truth",test_path="test",stdscore=0.32,result_roc='result_roc',image='image'):
    db=[]
    dd=[]
    conpos=0
    pos,db,temp=Analyze(standard_path,test_path)                     #get data from IOU

    if temp==0:
        print"there are not tag \"score\" in xml files" 
        return

    db = sorted(db, key=lambda x:x[2], reverse=True)#sorted()   
    
  
    for i in range(0,len(db)):                                       #select wrong and right 
        dbscore=db[i][2]
        #print dbscore
        if dbscore>=stdscore and db[i][0]==0 and db[i][1]==1:
            dd.append([0,1,dbscore])
            conpos+=1
        elif dbscore<stdscore and db[i][0]==1 and db[i][1]==0:
            dd.append([0,1,dbscore])
            conpos+=1
        elif dbscore<stdscore and db[i][0]==0 and db[i][1]==1:
            dd.append([1,0,dbscore])
        elif dbscore<stdscore and db[i][0]==0 and db[i][1]==1:
            dd.append([1,0,dbscore])
        else:
            '.'                                                 
    for i in range(len(dd)):                                        #random to draw picture
        a=random.random()
        dd[i][2]=round(a, 5)
    dd = sorted(dd, key=lambda x:x[2], reverse=True)#sorted()    
        
    xy_arr = []
    score =[]
    tp,fp = 0., 0.		
    for i in range(len(dd)):
        tp += dd[i][0]#wrong
        fp += dd[i][1]#right
        xy_arr.append([tp,fp/conpos])
        score.append(dd[i][2])
    #print score
    auc = 0.			
    prev_x = 0
    for x,y in xy_arr:
	    if x != prev_x:
		    auc += (x - prev_x) * y
		    prev_x = x     
         
    x = [_a[0] for _a in xy_arr]
    y = [_a[1] for _a in xy_arr]
    z = [_a for _a in score]
    
    #print z
    result_roc+='.txt'                                                 #parameter of picture
    image+='.png'
    plt.title("ROC (AUC = %.4f)" % (auc))
    plt.xlabel("False Count")
    plt.ylabel("True Positive Rate")
    plt.plot(x, y,color='blue',label='rate')
    plt.plot(x, z,color='black',label='score')
    xmajorLocator   = MultipleLocator(tp/8)
    xminorLocator   = MultipleLocator(tp/16) 
    ymajorLocator   = MultipleLocator(0.2)
    yminorLocator   = MultipleLocator(0.05) 
    
    ax = plt.subplot()
    ax.xaxis.set_major_locator(xmajorLocator)
    ax.yaxis.set_major_locator(ymajorLocator)
    ax.xaxis.set_minor_locator(xminorLocator)
    ax.yaxis.set_minor_locator(yminorLocator)
    ax.xaxis.grid(True, which='major')
    ax.yaxis.grid(True, which='minor')
    plt.legend(bbox_to_anchor=(0., 1.08, 1., .102), loc=2,ncol=2, mode='expand', borderaxespad=0.)
    plt.grid() 
    plt.grid(color='y' , linestyle='--')
    
    plt.savefig(image)
    plt.show()
    
    with open(result_roc, 'w') as fp:                                  #save datas
        for i in range(len(dd)):
            fp.write("%d %f %f \n" % (x[i], y[i],z[i]))
    
if __name__ == '__main__':
    roc()


# -*- coding: utf-8 -*-

from distutils.core import setup
from os import path
from setuptools import setup, find_packages


here = path.abspath(path.dirname(__file__))
with open(path.join(here,"README.txt"),"r") as fh:
    long_description=fh.read()

    
setup(

    name = 'roc2.1.3',

    version = '2.1.3',

    keywords = ('simple', 'test'),

    description = 'just a simple test of vipkid',
    
    long_description=long_description,
    
    long_description_content_type="text/markdown",
    charset='UTF-8',
    variant='GFM',

    license = 'MIT',

    author = 'yin',

    author_email = 'yinmengmeng@vipkid.com.cn',

    packages = find_packages(),

    platforms = 'any',

    py_modules=['ROC.drawpic','ROC.roc','ROC.DataOfRoc','ROC.IOU','ROC.scoreRoc','transform.txt2xml','transform.xml2txt','nms.nms']

)




# -*- coding: utf-8 -*-
import xml.dom.minidom
import os
import os.path

def txt2xml(pre_file='txt_file',after_file='xml_file'):
    line=[]
    temp=0
    files1 =os.listdir(pre_file)
    for xmlFile in files1:
        temp=temp+1
        if not os.path.isdir(xmlFile) and xmlFile.endswith('.txt'):#select txtfile
            with open(os.path.join(pre_file,xmlFile),'r') as fs:
                print "file %d is opened successfully"%temp
                managerList=[]
                doc = xml.dom.minidom.Document()#在内存中创建一个空的文档
                root = doc.createElement('annotation') #创建一个根节点Managers对象
                root.setAttribute('verified', 'no') #设置根节点的属性
                doc.appendChild(root)#将根节点添加到文档对象中
            
                nodedata=doc.createElement('folder')
                nodedata.appendChild(doc.createTextNode(after_file))
                root.appendChild(nodedata)
            
                nodedata=doc.createElement('filename')
                nodedata.appendChild(doc.createTextNode(xmlFile))
                root.appendChild(nodedata)
            
                nodedata=doc.createElement('path')
                nodedata.appendChild(doc.createTextNode(os.path.join(after_file,xmlFile)))
                root.appendChild(nodedata)
            
                nodedata=doc.createElement('source')
                nodeson=doc.createElement('database')
                nodeson.appendChild(doc.createTextNode('Unknown'))
                nodedata.appendChild(nodeson)
                root.appendChild(nodedata)
            
                nodedata=doc.createElement('size')
                nodewidth=doc.createElement('width')
                nodewidth.appendChild(doc.createTextNode('0'))
                nodeheight=doc.createElement('height')
                nodeheight.appendChild(doc.createTextNode('0'))
                nodedepth=doc.createElement('depth')
                nodedepth.appendChild(doc.createTextNode('0'))
                nodedata.appendChild(nodewidth)
                nodedata.appendChild(nodeheight)
                nodedata.appendChild(nodedepth)
                root.appendChild(nodedata)
            
                nodedata=doc.createElement('segmented')
                nodedata.appendChild(doc.createTextNode('0'))
                root.appendChild(nodedata)
        
                str=fs.read()
                line=str.split()
                length_line=len(line)-2
                
                
                for i in range(0,int(line[1])):
                    if (length_line/int(line[1]))==4:
                        managerList.append([{'xmin':line[2+i*4],'ymin':line[3+i*4],'xmax':line[4+i*4],'ymax':line[5+i*4]}])
                    elif (length_line/int(line[1]))==5:
                        managerList.append([{'xmin':line[2+i*5],'ymin':line[3+i*5],'xmax':line[4+i*5],'ymax':line[5+i*5],'score':line[6+i*5]}])
                    else:
                        print "file %s is wrong"%xmlFile
            
                for i in managerList :
                    for j in range(len(i)):
                        nodedata=doc.createElement('object')
                        nodename=doc.createElement('name')
                        nodename.appendChild(doc.createTextNode('NULL'))
                        nodepose=doc.createElement('pose')
                        nodepose.appendChild(doc.createTextNode('Unspecified'))
                        nodetruncated=doc.createElement('truncated')
                        nodetruncated.appendChild(doc.createTextNode('1'))
                        nodedifficult=doc.createElement('difficult')
                        nodedifficult.appendChild(doc.createTextNode('0'))
                        nodedata.appendChild(nodename)
                        nodedata.appendChild(nodepose)
                        nodedata.appendChild(nodetruncated)
                        nodedata.appendChild(nodedifficult)
                    
            
                        nodeManager = doc.createElement('bndbox')
                        nodeXmin = doc.createElement("xmin")
                        #给叶子节点name设置一个文本节点，用于显示文本内容
                        nodeXmin.appendChild(doc.createTextNode(i[j]['xmin']))
                        nodeYmin = doc.createElement("ymin")
                        nodeYmin.appendChild(doc.createTextNode(i[j]['ymin']))
                        nodeXmax = doc.createElement("xmax")
                        nodeXmax.appendChild(doc.createTextNode(i[j]['xmax']))
                        nodeYmax = doc.createElement("ymax")
                        nodeYmax.appendChild(doc.createTextNode(i[j]['ymax']))
        
                        #将各叶子节点添加到父节点Manager中，
                        #最后将Manager添加到根节点Managers中
                        nodeManager.appendChild(nodeXmin)
                        nodeManager.appendChild(nodeYmin)
                        nodeManager.appendChild(nodeXmax)
                        nodeManager.appendChild(nodeYmax)
                        if (length_line/int(line[1]))==5:
                            nodeScore = doc.createElement("score")                                   ########
                            nodeScore.appendChild(doc.createTextNode(i[j]['score']))
                            nodeManager.appendChild(nodeScore)
                        nodedata.appendChild(nodeManager)
                            
                    
                        root.appendChild(nodedata)
                #开始写xml文档
                if not(os.path.exists(after_file)):
                    os.mkdir(after_file)
                pathn=os.path.join(after_file,line[0])
                pathn+=".xml"
                fp = open(pathn, 'w')
                doc.writexml(fp, indent='\t', addindent='\t', newl='\n')
    print 'Transform_txt_xml() is OK'
        
if __name__ == '__main__':
    txt2xml()
            
    




# -*- coding: utf-8 -*-
import os
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from flv2jpg import avi2jpg

cnames = ('aquamarine','black','blue',
          'blueviolet','brown','burlywood','cadetblue','chartreuse','chocolate','coral','cornflowerblue',
          'crimson','cyan','darkblue','darkcyan','darkgoldenrod','darkgray','darkgreen','darkkhaki','darkmagenta',
          'darkolivegreen','darkorange','darkorchid','darkred','darkseagreen','darkslateblue','darkslategray',
          'darkturquoise','darkviolet','deeppink','deepskyblue','dimgray','firebrick','forestgreen',
          'fuchsia','gold','goldenrod','gray','green','hotpink','indianred',
          'indigo','lawngreen','lime','limegreen',
          'magenta','maroon','navy','olive','olivedrab',
          'orange','orchid','palevioletred','peru',
          'plum','purple','red','sienna','skyblue','slateblue','slategray','springgreen','steelblue',
          'tan','teal','thistle','turquoise','violet','yellow')

listdata=avi2jpg()
def roc(listdata=[],labela='data',image='image'):

    ab=listdata 
    #ab=drawpic()
    ab = sorted(ab, key=lambda x:x[2], reverse=True)#sorted() 
    mux=0
  
    for i in range(len(ab)):
        x = [_a[0] for _a in ab[i]]
        y = [_a[1] for _a in ab[i]]
        z = [_a[2] for _a in ab[i]]       
        plt.xlabel("False Count")
        plt.ylabel("True Positive Rate")
        plt.title("ROC")
        plt.plot(x, y,color=cnames[i%69],label=labela)
        plt.plot(x, z,color=cnames[i%69])
        for i in range(len(x)):
            if x[i]>mux:
                mux=x[i]
    image+='.jpg'
    
    
        
    xmajorLocator   = MultipleLocator(mux/8) 
    xminorLocator   = MultipleLocator(mux/16) 
    ymajorLocator   = MultipleLocator(0.2) 
    yminorLocator   = MultipleLocator(0.05) 
    
    ax = plt.subplot() 
    ax.xaxis.set_major_locator(xmajorLocator)
    ax.yaxis.set_major_locator(ymajorLocator)

    ax.xaxis.set_minor_locator(xminorLocator)
    ax.yaxis.set_minor_locator(yminorLocator)
    ax.xaxis.grid(True, which='major') 
    ax.yaxis.grid(True, which='minor') 
    
    plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=2,ncol=3, mode='expand', borderaxespad=0.)
    plt.grid() 
    plt.grid(color='y' , linestyle='--')
    
    plt.savefig(image)
    plt.show() 
    print "Run successfully"
    
    
if __name__ == '__main__':
    roc()

